/******************************************************************************
*
* Copyright (C) 2019 Allegro DVT2.  All rights reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Use of the Software is limited solely to applications:
* (a) running on a Xilinx device, or
* (b) that interact with a Xilinx device through a bus or interconnect.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
* XILINX OR ALLEGRO DVT2 BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
* OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* Except as contained in this notice, the name of  Xilinx shall not be used
* in advertising or otherwise to promote the sale, use or other dealings in
* this Software without prior written authorization from Xilinx.
*
*
* Except as contained in this notice, the name of Allegro DVT2 shall not be used
* in advertising or otherwise to promote the sale, use or other dealings in
* this Software without prior written authorization from Allegro DVT2.
*
******************************************************************************/

#include "SyncLog.h"

LogCategories::LogCategories() :
  categories{{ "framebuffer", true },
  {
    "channel", true
  },
  {
    "driver", true
  }}
{
}

LogCategories::~LogCategories() = default;

void printChannelStatus(ChannelStatus const& status)
{
  for(int fb = 0; fb < MAX_FB_NUMBER; ++fb)
  {
    for(int user = 0; user < MAX_USER; ++user)
      Log("channel", "%s[%d] Framebuffer %d = %s\n", (user == 0) ? "prod" : (user == 1) ? "cons" : "unknown", user, fb, status.fbAvail[fb][user] ? "available" : "busy");
  }

  Log("channel", "Enabled = %s\n", status.enable ? "true" : "false");
  Log("channel", "Sync Error = %s\n", status.syncError ? "true" : "false");
  Log("channel", "Wdg Error = %s\n", status.watchdogError ? "true" : "false");
  Log("channel", "Luma Diff Error = %s\n", status.lumaDiffError ? "true" : "false");
  Log("channel", "Chroma Diff Error = %s\n", status.chromaDiffError ? "true" : "false");
}

void printChannelStatus(int channelId, ChannelStatus const& status)
{
  Log("channel", "\n******** Channel %d status ***********\n", channelId);
  printChannelStatus(status);
  Log("channel", "*************************************\n");
}

void printAllChannelStatuses(SyncIp* syncIp)
{
  for(int i = 0; i < syncIp->maxChannels; ++i)
    printChannelStatus(i, syncIp->getStatus(i));
}

